from .tableview import TableView
from .exceptions import CloseBookException, EditNoteException
from .deletedialog import DeleteDialog
from .notepreview import NotePreview
import datetime
import os
import curses

from typing import Dict, Any

class NoteView(TableView):
    def __init__(self, stdscr, config: Dict[str, Any], book: str):
        self.book = book

        try: note_files = filter(lambda f: os.path.splitext(f)[1] == '.vmnt', os.scandir(os.path.join(config['notedir'], book)))
        except TypeError: pass
        except FileNotFoundError:
            os.makedirs(os.path.join(config['notedir'], book))
        else:
            self.content = [[
                os.path.splitext(note_file.name)[0],
                str(self._line_count(note_file)),
                datetime.datetime.fromtimestamp(os.stat(note_file).st_mtime_ns/1_000_000_000).strftime(config['dateformat'])] for note_file in note_files]

        height, width = stdscr.getmaxyx()
        self.preview = NotePreview(curses.newpad(round((height - 1) * config['previewratio']), width))
        self.empty_content_message = ['No notes detected.', 'Hit n to make one!']
        self.headers = ['NOTE TITLE (F1)  ', 'LINES (F2)  ', 'MODIFIED (F3)  '] # two spaces so there's room for an arrow when used for sorting
        self.keys = [
                lambda title:title,
                lambda count:int(count),
                lambda datestr:datetime.datetime.strptime(datestr, config['dateformat']) ]
        super().__init__(stdscr, config)

    def new(self, title: str):
        raise EditNoteException(self.book, title)

    def rename(self, row: int, new_name: str):
        os.rename(os.path.join(self.config['notedir'], self.book, self.content[row][0] + '.vmnt'), os.path.join(self.config['notedir'], self.book, new_name + '.vmnt'))
        self.content[row][0] = new_name

    def show_preview(self):
        self.update_preview()

    def update_preview(self):
        if len(self.content) == 0:
            return
        try: note_file = open(os.path.join(self.config['notedir'], self.book, self.content[self.real_selected][0] + '.vmnt'))
        except FileNotFoundError: return
        self.preview.update(self.real_selected, note_file)

    def show_delete_dialog(self, row: int):
        size_cutoff = round(self.stdscr.getmaxyx()[1]/2) - 21
        if len(title := self.content[row][0]) > size_cutoff:
            title = title[:size_cutoff] + '…'
        if len(book_title := self.book) > size_cutoff:
            book_title = book_title[:size_cutoff] + '…'
        self.delete_dialog = DeleteDialog(['Are you sure you want to delete', f'note "{title}" from book "{book_title}"?', 'This cannot be undone.'])

    def delete(self, row: int):
        title = self.content.pop(row)[0]
        os.remove(os.path.join(self.config['notedir'], self.book, title + '.vmnt'))

    @staticmethod
    def _line_count(file):
        with open(file) as f:
            return len(f.readlines())

    def draw(self):
        self.stdscr.addstr(0,0, f'Book: {self.book}')
        self.stdscr.clrtoeol()
        super().draw()

    def on_enter(self, row: int):
        title = self.content[row][0]
        raise EditNoteException(self.book, title)

    def on_escape(self):
        raise CloseBookException



from .preview import Preview
import io # for type hinting
from typing import List
import logging
import textwrap

class NotePreview(Preview):
    def __init__(self, pad):
        super().__init__(pad)
        self.content: List[str] = []

    def update(self, row: int, note_file: io.TextIOWrapper):
        super().update(row)
        with note_file:
            # split the file by newlines and by text wrap
            self.content = [line for rawline in note_file.readlines()[:self.pad.getmaxyx()[0] - 2]
                    for line in textwrap.fill(rawline[:-1], self.pad.getmaxyx()[1] - 2).split('\n')]



import configparser
import os

def get_config(path: str):
    # config file doesn't have sections but parser requires sections
    # so just add a default section after loading file
    try:
        f = open(path)
    except IOError:
        content = []
    else:
        with f:
            content = f.readlines()
    content.insert(0, '[DEFAULT]')
    cfg = configparser.ConfigParser()
    cfg.read_string('\n'.join(content))

    # configparser doesn't automatically guess the datatypes based on fallback value
    # so we create a list associating keys, getters, and fallbacks
    # and then use the parser to fill in what it can
    default = cfg['DEFAULT']
    options = [
            ('previewratio', default.getfloat, 0.5),
            ('confirmdelete', default.getboolean, True),
            ('notedir', lambda path, *args, **kwargs: os.path.expanduser(default.get(path, *args, **kwargs)), os.path.expanduser('~/.vimnote/')),
            ('dateformat', default.get, '%I:%M%p %m-%d-%Y'),
            ('defaultsortcol', default.getint, 2),
            ('defaultsortascending', default.getboolean, True) ]
    return { key: getter(key, fallback=fallback) for (key, getter, fallback) in options }
